import pytest

from elasticsearch_tools import query as q


@pytest.mark.parametrize(
    "query, function_kwargs",
    [
        (q.ElasticQueryString(field="f", value="v"), {"_type": "query_string", "field": "f", "value": "v"}),
        (q.ElasticQueryString(field="f"), {"_type": "query_string", "field": "f"}),
        (q.ElasticQueryString(value="v"), {"_type": "query_string", "value": "v"}),
        (q.ElasticQueryString(), {"_type": "query_string"}),
        (q.ElasticTermQuery(field="f", value="v"), {"_type": "term", "field": "f", "value": "v"}),
        (q.ElasticTermQuery(field="f", value='"v"'), {"_type": "term", "field": "f", "value": '"v"'}),
        (q.ElasticFuzzyQuery(field="f", value="v"), {"_type": "fuzzy", "field": "f", "value": '"v"'}),
        (q.ElasticFuzzyQuery(field="f", value='"v"'), {"_type": "fuzzy", "field": "f", "value": "v"}),
        (q.ElasticExistsQuery(field="f"), {"_type": "exists", "field": "f"}),
        (q.ElasticFullMatchQuery(field="f", value="v"), {"_type": "match_phrase", "field": "f", "value": "v"}),
        (q.ElasticFullMatchQuery(field="f", value="v", boosting=1), {"_type": "match_phrase", "field": "f", "value": "v", "boosting": 1}),
        (q.ElasticMatchQuery(field="f", value="v"), {"_type": "match", "field": "f", "value": "v"}),
        (q.ElasticMatchQuery(field="f", value="v", boosting=1), {"_type": "match", "field": "f", "value": "v", "boosting": 1}),
        (q.ElasticRangeQuery(field="f", value_from=0, value_to=1), {"_type": "range", "field": "f", "value_from": 0, "value_to": 1}),
        (q.ElasticGeoPointRangeQuery(field="f", value_from="[2, 3]", value_to="[3, 4]"), {"_type": "geo_point_range", "field": "f", "value_from": "[2, 3]", "value_to": "[3, 4]"}),
        (q.ElasticGeoPointQuery(field="f", value="[1,1]"), {"_type": "geo_point", "field": "f", "value": "[1,1]"}),
    ],
)
def test_query_writing(query, function_kwargs):
    new_query = q.generate_query(**function_kwargs)
    assert new_query.get_query() == query.get_query()



import pytest

from elasticsearch_tools import query as q


@pytest.mark.parametrize(
    "auto_query, query",
    [
        (q.ElasticQueryString(field="f", value="v"), {"query_string": {"default_field": "f", "query": "v"}}),
        (q.ElasticQueryString(field="f"), {"query_string": {"default_field": "f", "query": "*"}}),
        (q.ElasticQueryString(value="v"), {"query_string": {"default_field": "*", "query": "v"}}),
        (q.ElasticQueryString(), {"query_string": {"default_field": "*", "query": "*"}}),
        (q.ElasticTermQuery(field="f", value="v"), {"term": {"f": "v"}}),
        (q.ElasticTermQuery(field="f", value='"v"'), {"term": {"f": "v"}}),
        (q.ElasticFuzzyQuery(field="f", value="v"), {"fuzzy": {"f": "v"}}),
        (q.ElasticFuzzyQuery(field="f", value='"v"'), {"fuzzy": {"f": "v"}}),
        (q.ElasticExistsQuery(field="f"), {"exists": {"field": "f"}}),
        (q.ElasticFullMatchQuery(field="f", value="v"), {"match_phrase": {"f": "v"}}),
        (q.ElasticFullMatchQuery(field="f", value="v", boosting=1), {"match_phrase": {"f": "v", "boosting": 1}}),
        (q.ElasticFullMatchQuery(field="f", value='"v"'), {"match_phrase": {"f": "v"}}),
        (q.ElasticMatchQuery(field="f", value="v"), {"match": {"f": "v"}}),
        (q.ElasticMatchQuery(field="f", value="v", boosting=1), {"match": {"f": "v", "boosting": 1}}),
        (q.ElasticMatchQuery(field="f", value='"v"'), {"match": {"f": "v"}}),
        (q.ElasticRangeQuery(field="f", value_from=0, value_to=1), {"range": {"f": {"gte": 0, "lte": 1}}}),
        (q.ElasticRangeQuery(field="f", value_from=1, value_to=0), {"range": {"f": {"gte": 0, "lte": 1}}}),
        (q.ElasticGeoPointRangeQuery(field="f", value_from="[2, 3]", value_to="[3, 4]"), {"geo_bounding_box": {"location.coordinates": {"bottom_right": {"lat": "3", "lon": " 4"}, "top_left": {"lat": "2", "lon": " 3"}}}}),
        (q.ElasticGeoPointQuery(field="f", value="[1,1]"), {"geo_bounding_box": {"location.coordinates": {"bottom_right": {"lat": "1", "lon": "1"}, "top_left": {"lat": "1", "lon": "1"}}}}),
        (q.ElasticNestedQuery(query=q.ElasticExistsQuery(field="f"), nested_path="core"), {"nested": {"path": "core", "query": {"exists": {"field": "f"}}}})
    ],
)
def test_query_writing(auto_query, query):
    assert auto_query.get_query() == query

@pytest.mark.parametrize(
    "query",
    [
        q.ElasticQueryString(),
        q.ElasticTermQuery(field="f", value="v"),
        q.ElasticFuzzyQuery(field="f", value="v"),
        q.ElasticExistsQuery(field="f"),
        q.ElasticFullMatchQuery(field="f", value="v"),
        q.ElasticMatchQuery(field="f", value="v"),
        q.ElasticRangeQuery(field="f", value_from=0, value_to=1),
        q.ElasticGeoPointRangeQuery(field="f", value_from="[2, 3]", value_to="[3, 4]"),
        q.ElasticGeoPointQuery(field="f", value="[1,1]"),
        q.ElasticNestedQuery(query=q.ElasticExistsQuery(field="f"), nested_path="core")
    ],
)
def test_query_bool(query):
    hardcode_and = q.ElasticBoolMust(queries=[query, query])
    hardcode_or = q.ElasticBoolShould(queries=[query, query])

    auto_and = query & query
    assert auto_and.get_query() == hardcode_and.get_query()

    auto_rand = query
    auto_rand &= query
    assert auto_and.get_query() == hardcode_and.get_query()

    auto_or = query | query
    assert auto_or.get_query() == hardcode_or.get_query()

    auto_ror = query
    auto_ror |= query
    assert auto_ror.get_query() == hardcode_or.get_query()




from elasticsearch_tools.connection.async_client.connection import (
    elastic_async_db_manager,
    get_async_elastic_client,
    get_async_elastic_client_generator,
)
from elasticsearch_tools.connection.connection import (
    elastic_db_manager,
    get_elastic_client,
    get_elastic_client_generator,
)
from elasticsearch_tools.query.base import ElasticBaseQuery
from elasticsearch_tools.query.bool import ElasticBoolMust, ElasticBoolMustNot, ElasticBoolQuery, ElasticBoolShould
from elasticsearch_tools.query.search import (
    ElasticExistsQuery,
    ElasticFullMatchQuery,
    ElasticFuzzyQuery,
    ElasticGeoPointQuery,
    ElasticGeoPointRangeQuery,
    ElasticMatchQuery,
    ElasticNestedQuery,
    ElasticQueryString,
    ElasticRangeQuery,
    ElasticSearchQuery,
    ElasticTermQuery,
)

from elasticsearch_tools.query import generate_query



import pytest

from elasticsearch_tools.query import ElasticBoolMust, ElasticBoolMustNot, ElasticBoolShould, ElasticFuzzyQuery


@pytest.mark.parametrize(
    "query_class, negative_query_class",
    [
        (ElasticBoolMustNot, ElasticBoolMust),
        (ElasticBoolMust, ElasticBoolMustNot)
    ],
)
def test_denial(query_class, negative_query_class):
    query = query_class([])
    reversed_negative_query = ~negative_query_class([])
    assert query.get_query() == reversed_negative_query.get_query()


def test_denial_should():
    auto_should_not_query = ~ElasticBoolShould([])
    should_not_query = ElasticBoolMustNot([ElasticBoolShould([])])
    assert auto_should_not_query.get_query() == should_not_query.get_query()


@pytest.mark.parametrize(
    "query_class, query",
    [
        (ElasticBoolMustNot, {"bool": {"must_not": []}}),
        (ElasticBoolMust, {"bool": {"must": []}}),
        (ElasticBoolShould, {"bool": {"should": []}})
    ],
)
def test_query_writing(query_class, query):
    auto_query = query_class([])
    assert auto_query.get_query() == query


@pytest.mark.parametrize(
    "query_class, query",
    [
        (ElasticBoolMustNot, {'bool': {'should': [{'bool': {'must_not': []}}, {'bool': {'must_not': []}}]}}),
        (ElasticBoolMust, {"bool": {"should": [{"bool": {"must": []}}, {"bool": {"must": []}}]}}),
        (ElasticBoolShould, {"bool": {"should": []}})
    ],
)
def test_query_or(query_class, query):
    auto_query = query_class([])
    new_query = auto_query | auto_query
    assert new_query.get_query() == query

    new_query = query_class([])
    new_query |= auto_query
    assert new_query.get_query() == query


@pytest.mark.parametrize(
    "query_class, query",
    [
        (ElasticBoolMustNot, {'bool': {'must': [{'bool': {'must_not': []}}, {'bool': {'must_not': []}}]}}),
        (ElasticBoolMust, {"bool": {"must": []}}),
        (ElasticBoolShould, {"bool": {"must": [{"bool": {"should": []}}, {"bool": {"should": []}}]}})
    ],
)
def test_query_and(query_class, query):
    auto_query = query_class([])
    new_query = auto_query & auto_query
    assert new_query.get_query() == query

    new_query = query_class([])
    new_query &= auto_query
    assert new_query.get_query() == query


@pytest.mark.parametrize(
    "query_class, query",
    [
        (ElasticBoolMustNot, {"bool": {"must": []}}),
        (ElasticBoolMust, {"bool": {"must_not": []}}),
        (ElasticBoolShould, {"bool": {"must_not": [{"bool": {"should": []}}]}})
    ],
)
def test_query_not(query_class, query):
    auto_query = ~query_class([])
    assert auto_query.get_query() == query


@pytest.mark.parametrize(
    "query_class",
    [
        ElasticBoolMustNot,
        ElasticBoolMust,
        ElasticBoolShould
    ],
)
def test_query_getitem(query_class):
    query = query_class([ElasticFuzzyQuery(field="f", value="v")])
    first_query = query[0]
    assert isinstance(first_query, ElasticFuzzyQuery)
    assert first_query.get_query() == {"fuzzy": {"f": "v"}}